package com.zehong.communication.common.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.zehong.communication.common.result.Result;
import com.zehong.communication.common.result.ResultEnum;
import com.zehong.communication.entity.RequestLog;
import com.zehong.communication.model.BaseRequestModel;
import com.zehong.communication.service.RequestLogService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.UUID;

@Order(0)
@Aspect
@Component
public class LogInterceptor {

    private static final Logger logger = LoggerFactory.getLogger(LogInterceptor.class);

    @Autowired
    private RequestLogService requestLogService;

    @Value("${secretKey}")
    private String secretKey;

    /**
     * 定义拦截器规则
     */

    @Pointcut("execution(* com.zehong.communication.controller.*.*(..))")
    public void pointCut() {
    }

    /**
     * 拦截器具体实现
     *
     */
    @Around(value = "pointCut()")
    public Object around(ProceedingJoinPoint point){
        // 获取方法参数
        Object[] pointParams = point.getArgs();
        String params = Arrays.toString(pointParams);
        try{
            for(Object pointParam : pointParams){
                if(pointParam instanceof BaseRequestModel){
                    String secretKey = ((BaseRequestModel) pointParam).getSecretKey();
                    if(!this.secretKey.equals(secretKey)){
                        return new Result(ResultEnum.SECRETKRY_ERROR);
                    }
                }
            }
            Object result = point.proceed();
            insertRequestLogRecord(params,JSONObject.toJSONString(result));
            return result;
        }catch (Throwable e){
            logger.error("日志拦截出错："+e);
            insertRequestLogRecord(params,e.getMessage());
            return new Result(ResultEnum.SYSTEM_EXCEPTION);
        }
    }

    /**
     * 记录请求日志
     * @param params 请求参数
     * @param result 请求结果
     */
    private void insertRequestLogRecord(String params,String result){
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        request.getParameter("secretKey");
        RequestLog requestLog = new RequestLog();
        requestLog.setId(UUID.randomUUID().toString().replaceAll("-",""));
        requestLog.setRequestMapping(request.getRequestURL().toString());
        requestLog.setParams(params);
        requestLog.setSecretKey(secretKey);
        requestLog.setPostTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        requestLog.setResult(JSONObject.toJSONString(result));
        requestLogService.insert(requestLog);
    }
}
